/**
 * Created by Mihail on 5/30/18.
 */
'use strict';

const fetch = require('node-fetch');
const FormData = require('form-data');
const qs = require('qs');
const request = require('request');

class Hasoffers {
    constructor({networkId = "", networkToken = "", fetcher = fetch, config = {}} = {}) {
        this.networkId = networkId;
        this.networkToken = networkToken;
        this.config = Object.assign({}, {
            method: 'GET',
            headers: {
                Accept: 'application/json',
                'Accept-Charset': 'utf-8'
            }
        }, config);

        this.fetcher = fetcher;
        this.url = `https://${this.networkId}.api.hasoffers.com`;
    }

    getOffer(id) {
        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Offer&Method=findById&id=${id}`,
            config: this.config
        })
    }

    getAllCountries() {
        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Application&Method=findAllCountries`,
            config: this.config
        })
    }

    createOffer(data) {
        let queryString = Object.keys(data).map(key => `data[${key}]` + '=' + encodeURIComponent(data[key])).join('&');
        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Offer&Method=create&${queryString}`,
            config: this.config
        })
    }

    updateOfferField(id, field, value) {
        let queryValue = encodeURIComponent(value);
        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Offer&Method=updateField&id=${id}&field=${field}&value=${queryValue}`,
            config: this.config
        })
    }

    updateOffer(id, data) {
        let queryString = Object.keys(data).map(key => `data[${key}]` + '=' + encodeURIComponent(data[key])).join('&');
        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Offer&Method=update&id=${id}&${queryString}`,
            config: this.config
        })
    }

    createOfferTag(offer_id, tag_id) {
        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Tag&Method=addToOffer&&offer_id=${offer_id}&tag_id=${tag_id}`,
            config: this.config
        })
    }

    createGeoCountries(id, data_country, data_city) {
        let queryCountries = Object.keys(data_country).map(key => `location_ids[Countries][]` + '=' + encodeURIComponent(data_country[key])).join('&');
        let queryCities = Object.keys(data_city).map(key => `location_ids[Cities][]` + '=' + encodeURIComponent(data_city[key])).join('&');

        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Offer&Method=setGeoTargeting&id=${id}&${queryCountries}&${queryCities}&exclude[]=cities`,
            config: this.config
        })
    }

    createGeoCities(id, data) {
        let queryString = Object.keys(data).map(key => `location_ids[Cities][]` + '=' + encodeURIComponent(data[key])).join('&');

        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Offer&Method=setGeoTargeting&id=${id}&${queryString}`,
            config: this.config
        })
    }

    createGeoCitiesExclude(id, data) {
        let queryString = Object.keys(data).map(key => `location_ids[Cities][]` + '=' + encodeURIComponent(data[key])).join('&');

        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=Offer&Method=setGeoTargeting&id=${id}&${queryString}&exclude[]=cities`,
            config: this.config
        })
    }

    createOfferFile(offer_id, type, filename, display, url) {
        const form = new FormData();
        form.append('file', request(url));

        return createRequest({
            fetcher: this.fetcher,
            url: `${this.url}/Apiv3/json?NetworkToken=${this.networkToken}&Target=OfferFile&Method=create&data[offer_id]=${offer_id}&data[type]=${type}&data[display]=${display}&data[file][name]=${filename}&return_object=1`,
            config: {
                method: 'POST',
                body: form
            }
        })
    }
}
const createRequest = (args = {}) => {
    const {url, config, query, fetcher} = args;
    return fetch(`${url}${query ? `?${qs.stringify(query)}` : ''}`, config).then(res =>
        res.json()
    )
};

module.exports = Hasoffers;